#!/bin/bash
#
# if this build script complains that it cannot find bzlib.h, 
# you need to install bzip2-devel on Scientific Linux

usage()
{
   echo "USAGE: `basename ${0}` <product_dir> <i1|e7|e9|e10|e11> <debug|opt|prof> [tar]"
}

# -------------------------------------------------------------------
# shared boilerplate
# -------------------------------------------------------------------

get_this_dir() 
{
    ( cd / ; /bin/pwd -P ) >/dev/null 2>&1
    if (( $? == 0 )); then
      pwd_P_arg="-P"
    fi
    reldir=`dirname ${0}`
    thisdir=`cd ${reldir} && /bin/pwd ${pwd_P_arg}`
}

get_ssibuildshims()
{
    # make sure we can use the setup alias
    if [ -z ${UPS_DIR} ]
    then
       echo "ERROR: please setup ups"
       exit 1
    fi
    source `${UPS_DIR}/bin/ups setup ${SETUP_UPS}`
    
    setup ssibuildshims ${ssibuildshims_version} -z ${product_dir}
}

set_qualifiers()
{
 
    if [ -z ${basequal} ]
    then
       echo "ERROR: please specify the qualifiers"
       usage
       exit 1
    fi

    fullqual=${basequal}:${extraqual}
    qualdir=`echo ${basequal} | sed -e 's/:/-/g'`-${extraqual}
    if [ "${extraqual}" = "opt" ]
    then
	qualver=`echo ${basequal} | sed -e 's/:/_/g'`_${extraqual}
    elif [ "${extraqual}" = "debug" ]
    then
	qualver=${extraqual}_`echo ${basequal} | sed -e 's/:/_/g'`
    elif [ "${extraqual}" = "prof" ]
    then
	qualver=`echo ${basequal} | sed -e 's/:/_/g'`_${extraqual}
    else
       echo "ERROR: please specify debug, opt, or prof"
       usage
       exit 1
    fi
}

get_platform_name()
{
    OS1=`uname`
    flvr=`ups flavor`
    if [ ${OS1} = "Darwin" ]
    then
	flvr=`ups flavor -2`
    fi
    OS=`${SSIBUILDSHIMS_DIR}/bin/get_os_platform`
    plat=`${SSIBUILDSHIMS_DIR}/bin/get_os_platform osplat`

    if [ -z ${flvr} ]
    then
       echo "ERROR: could not determine flavor for this platform"
       echo "       something is very wrong"
       exit 1
    fi

    pkgdir=${product_dir}/${package}/${pkgver}/${flvr}-${qualdir}
    patchdir=${product_dir}/${package}/${pkgver}/patch
    blddir=${product_dir}/${package}/${pkgver}/build/${flvr}-${qualdir}
    tardir=${product_dir}/${package}/${pkgver}/tar
    sourcedir=${product_dir}/${package}/${pkgver}/source
}

die()
{
  echo "$1"; exit 2
}

# -------------------------------------------------------------------
# start processing
# -------------------------------------------------------------------

product_dir=${1}
basequal=${2}
extraqual=${3}
maketar=${4}

if [ -z ${product_dir} ]
then
   echo "ERROR: please specify the local product directory"
   usage
   exit 1
fi

# -------------------------------------------------------------------
# package name and version
# -------------------------------------------------------------------

package=boost
origpkgver=v1_61_0
pkgver=${origpkgver}a
ssibuildshims_version=v0_09_02
pkgdotver=`echo ${pkgver} | sed -e 's/_/./g' | sed -e 's/^v//'`
pkgtarver=`echo ${origpkgver} | sed -e 's/^v//'`
pkgtarname=boost_${pkgtarver}

get_this_dir

get_ssibuildshims

set_qualifiers

get_platform_name

if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
   exit 0
fi

echo "building ${package} for ${OS}-${plat}-${qualdir} (flavor ${flvr})"

# -------------------------------------------------------------------
# package dependent stuff goes here
# -------------------------------------------------------------------

if [ "${extraqual}" = "opt" ]
then
    extra_command="variant=release"
elif [ "${extraqual}" = "debug" ]
then
    extra_command="variant=debug"
elif [ "${extraqual}" = "prof" ]
then
    extra_command="variant=profile"
fi
case ${basequal} in
    e7)
        cxxflg="-gdwarf-2 -std=c++1y -Wno-deprecated-declarations"
        ;;
    e9|e1[01])
        cxxflg="-std=c++14 -Wno-deprecated-declarations"
        ;;
    i1)
        cxxflg="-gdwarf-2 -std=c++11"
        ;;
    *)
        die "Unrecognized basequal ${basequal}"
esac

if [ ${OS1} = "Darwin" ]
then
  cxxflg="${cxxflg} -fvisibility-inlines-hidden"
fi

mkdir -p ${pkgdir}
if [ ! -d ${pkgdir} ]
then
   echo "ERROR: failed to create ${pkgdir}"
   exit 1
fi
mkdir -p ${blddir}
if [ ! -d ${blddir} ]
then
   echo "ERROR: failed to create ${blddir}"
   exit 1
fi
mkdir -p ${sourcedir}
if [ ! -d ${sourcedir} ]
then
   echo "ERROR: failed to create ${sourcedir}"
   exit 1
fi

# declare now so we can setup
# fake ups declare
fakedb=${product_dir}/${package}/${pkgver}/fakedb
${SSIBUILDSHIMS_DIR}/bin/fake_declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

setup -B ${package} ${pkgver} -q ${fullqual} -z ${fakedb}:${PRODUCTS} || die "ERROR: fake setup failed"

set -x


# this is for building
cd "${blddir}"
tar xf "${tardir}/${pkgtarname}.tar.bz2"
cd "${pkgtarname}"

# this will be distributed for debugging
cd "${sourcedir}"
# Make sure the source code directory tree is readable
tar xf "${tardir}/${pkgtarname}.tar.bz2" && \
  find "${pkgtarname}" -type d -exec chmod 0755 \{\} \;

cd "${pkgtarname}"

cd "${blddir}/${pkgtarname}"
./bootstrap.sh --prefix="${pkgdir}" --without-icu --without-libraries=python,mpi

if [ ${OS1} = "Darwin" ]
then
  mv project-config.jam project-config.jam.orig
  sed s%GCCDIR%"${GCC_FQ_DIR}"%g "${patchdir}/Darwin-project-config.jam" | sed s%BOOSTDIR%"${pkgdir}"%g > "${blddir}/${pkgtarname}/project-config.jam"
fi

num_cores=$(${SSIBUILDSHIMS_DIR}/bin/ncores)

# --debug-building  provides verbose output
# -d+2 echoes build commands
if [ -z "${cxxflg}" ]
then
  ./b2 -j ${num_cores} ${useicc:+"toolset=intel"} "${extra_command}"
else
  ./b2 -j ${num_cores} ${useicc:+"toolset=intel"} cxxflags="${cxxflg}" "${extra_command}"
fi
(( $? == 0 )) || die "ERROR: Build failed."

./b2 -j ${num_cores} ${useicc:+"toolset=intel"} "${extra_command}" install || die "ERROR: install failed."

set +x

if [ ! -d ${pkgdir}/lib ]
then
   echo "ERROR: failed to create ${pkgdir}/lib"
   echo "       Something is very wrong"
   exit 1
fi

# real ups declare
"${SSIBUILDSHIMS_DIR}/bin/declare_product" "${product_dir}" ${package} ${pkgver} ${fullqual}

# -------------------------------------------------------------------
# common bottom stuff
# -------------------------------------------------------------------

# this should not complain
echo "Finished building ${package} ${pkgver}"
setup ${package} ${pkgver} -q ${fullqual} -z ${product_dir}:${PRODUCTS}
echo "${package} is installed at ${BOOST_FQ_DIR}"
echo "BOOST_LIB is ${BOOST_LIB}"

# this must be last
if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
fi

exit 0

